@@ -21,6 +21,7 @@
 #include "persistentstorage.h"
 #include "services.h"
 #include <cmath>
+#include <cassert>
 extern "C" {
 #include "common.h"
 }
@@ -230,6 +231,146 @@ void Object::removeChangeListener(ChangeListener* listener)
     listenerList_m.remove(listener);
 }
 
+SwitchingObjectValue::SwitchingObjectValue(const std::string& value)
+{
+    if (value == "1" || value == "on" || value == "true")
+        value_m = true;
+    else if (value == "0" || value == "off" || value == "false")
+        value_m = false;
+    else
+    {
+        std::stringstream msg;
+        msg << "SwitchingObjectValue: Bad value: '" << value << "'" << std::endl;
+        throw ticpp::Exception(msg.str());
+    }
+}
+
+std::string SwitchingObjectValue::toString()
+{
+	return value_m ? "on" : "off";
+}
+
+DimmingObjectValue::DimmingObjectValue(const std::string& value)
+{
+    if (value == "stop")
+        value_m = 0;
+    else if (value == "up")
+        value_m = 9;
+    else if (value == "down")
+        value_m = 1;
+    else
+    {
+        std::stringstream msg;
+        msg << "DimmingObjectValue: Bad value: '" << value << "'" << std::endl;
+        throw ticpp::Exception(msg.str());
+    }
+}
+    
+std::string DimmingObjectValue::toString()
+{
+	switch (value_m)
+	{
+	case 0:
+		return "stop";
+	case 1:
+		return "down";
+	case 9:
+		return "up";
+	}
+	return "";
+}
+
+TimeObjectValue::TimeObjectValue(const std::string& value) : hour_m(-1), min_m(-1), sec_m(-1), wday_m(-1) 
+{
+	if (value == "now")
+		return;
+    std::istringstream val(value);
+   	char s1, s2;
+    val >> hour_m >> s1 >> min_m >> s2 >> sec_m;
+   	wday_m = 0;
+
+    if ( val.fail() || s1 != ':' || s2 != ':' )
+   	{
+       	std::stringstream msg;
+        msg << "TimeObjectValue: Bad value: '" << value << "'" << std::endl;
+   	    throw ticpp::Exception(msg.str());
+    }
+}
+    
+std::string TimeObjectValue::toString()
+{
+	if (hour_m == -1)
+		return "now";
+	std::ostringstream out;
+	out << hour_m << ":" << min_m << ":" << sec_m;
+	return out.str();
+}
+
+DateObjectValue::DateObjectValue(const std::string& value) : year_m(-1), month_m(-1), day_m(-1)
+{
+	if (value == "now")
+		return;
+    std::istringstream val(value);
+    char s1, s2;
+    val >> year_m >> s1 >> month_m >> s2 >> day_m;
+
+    if ( val.fail() || s1 != '-' || s2 != '-' )
+    {
+        std::stringstream msg;
+        msg << "DateObjectValue: Bad value: '" << value << "'" << std::endl;
+        throw ticpp::Exception(msg.str());
+    }
+}
+    
+std::string DateObjectValue::toString()
+{
+	if (day_m == -1)
+		return "now";
+	std::ostringstream out;
+	out << year_m << "-" << month_m << "-" << day_m;
+	return out.str();
+}
+
+ValueObjectValue::ValueObjectValue(const std::string& value)
+{
+    std::istringstream val(value);
+    val >> value_m;
+
+    if ( val.fail() )
+    {
+        std::stringstream msg;
+        msg << "ValueObjectValue: Bad value: '" << value << "'" << std::endl;
+        throw ticpp::Exception(msg.str());
+    }
+}
+    
+std::string ValueObjectValue::toString()
+{
+	std::ostringstream out;
+	out << value_m;
+	return out.str();
+}
+
+ScalingObjectValue::ScalingObjectValue(const std::string& value)
+{
+    std::istringstream val(value);
+    val >> value_m;
+
+    if ( val.fail() )
+    {
+        std::stringstream msg;
+        msg << "ScalingObjectValue: Bad value: '" << value << "'" << std::endl;
+        throw ticpp::Exception(msg.str());
+    }
+}
+    
+std::string ScalingObjectValue::toString()
+{
+	std::ostringstream out;
+	out << value_m;
+	return out.str();
+}
+
 SwitchingObject::SwitchingObject() : value_m(false)
 {
 }
@@ -238,6 +379,33 @@ SwitchingObject::~SwitchingObject()
 {
 }
 
+ObjectValue* SwitchingObject::createObjectValue(const std::string& value)
+{
+    return new SwitchingObjectValue(value);
+}
+
+bool SwitchingObject::equals(ObjectValue* value)
+{
+    assert(value);
+    SwitchingObjectValue* val = dynamic_cast<SwitchingObjectValue*>(value);
+    if (val == 0)
+    {
+        std::cout << "SwitchingObject: ERROR, equals() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+        return false;
+    }
+    std::cout << "SwitchingObject (id=" << getID() << "): Compare value_m='" << value_m << "' to value='" << val->value_m << "'" << std::endl;
+    return value_m == val->value_m; 
+}
+
+void SwitchingObject::setValue(ObjectValue* value)
+{
+    assert(value);
+    SwitchingObjectValue* val = dynamic_cast<SwitchingObjectValue*>(value);
+    if (val == 0)
+        std::cout << "SwitchingObject: ERROR, setValue() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+	setBoolValue(val->value_m);	
+}
+
 void SwitchingObject::onWrite(const uint8_t* buf, int len)
 {
     bool newValue;
@@ -282,6 +450,42 @@ void DimmingObject::exportXml(ticpp::Element* pConfig)
     pConfig->SetAttribute("type", "EIS2");
 }
 
+ObjectValue* DimmingObject::createObjectValue(const std::string& value)
+{
+    return new DimmingObjectValue(value);
+}
+
+bool DimmingObject::equals(ObjectValue* value)
+{
+    assert(value);
+    DimmingObjectValue* val = dynamic_cast<DimmingObjectValue*>(value);
+    if (val == 0)
+    {
+        std::cout << "DimmingObject: ERROR, equals() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+        return false;
+    }
+    std::cout << "DimmingObject (id=" << getID() << "): Compare value_m='" << value_m << "' to value='" << val->value_m << "'" << std::endl;
+	int stepcode = value_m & 0x7;
+	int intval;
+	// only consider 3 different values: up/down/stop 
+	if (stepcode == 0)
+		intval = 0;
+	else if (value_m > 8)
+		intval = 9;   
+	else
+		intval = 1;   
+    return intval == val->value_m; 
+}
+
+void DimmingObject::setValue(ObjectValue* value)
+{
+    assert(value);
+    DimmingObjectValue* val = dynamic_cast<DimmingObjectValue*>(value);
+    if (val == 0)
+        std::cout << "DimmingObject: ERROR, setValue() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+	setIntValue(val->value_m);	
+}
+
 void DimmingObject::onWrite(const uint8_t* buf, int len)
 {
     int newValue;
@@ -328,6 +532,36 @@ void TimeObject::exportXml(ticpp::Element* pConfig)
     pConfig->SetAttribute("type", "EIS3");
 }
 
+ObjectValue* TimeObject::createObjectValue(const std::string& value)
+{
+    return new TimeObjectValue(value);
+}
+
+bool TimeObject::equals(ObjectValue* value)
+{
+    assert(value);
+    TimeObjectValue* val = dynamic_cast<TimeObjectValue*>(value);
+    if (val == 0)
+    {
+        std::cout << "TimeObject: ERROR, equals() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+        return false;
+    }
+//    std::cout << "TimeObject (id=" << getID() << "): Compare value_m='" << value_m << "' to value='" << val->value_m << "'" << std::endl;
+    return (sec_m == val->sec_m) && (min_m == val->min_m) && (hour_m == val->hour_m) && (wday_m == val->wday_m); 
+}
+
+void TimeObject::setValue(ObjectValue* value)
+{
+    assert(value);
+    TimeObjectValue* val = dynamic_cast<TimeObjectValue*>(value);
+    if (val == 0)
+        std::cout << "TimeObject: ERROR, setValue() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+	if (val->hour_m == -1)
+		setTime(time(0));
+	else        
+		setTime(val->wday_m, val->hour_m, val->min_m, val->sec_m);	
+}
+
 void TimeObject::onWrite(const uint8_t* buf, int len)
 {
     if (len < 5)
@@ -354,32 +588,6 @@ void TimeObject::onWrite(const uint8_t* buf, int len)
     }
 }
 
-void TimeObject::setIntValue(int value)
-{
-    if (value < 0)
-        setTime(time(0));
-    else
-    {
-        int rem = value;
-        int sec = rem % 60;
-        rem = (rem-sec) / 60; 
-        int min = rem % 60;
-        rem = (rem-min) / 60; 
-        int hour = rem % 24;
-        rem = (rem-hour) / 24; 
-        int wday = rem % 8;
-
-        setTime(wday, hour, min, sec);
-    }
-}
-
-int TimeObject::getIntValue()
-{
-    if (!init_m)
-        read();
-    return ((wday_m*24 + hour_m) * 60 + min_m) * 60 + sec_m;
-}
-
 void TimeObject::setTime(time_t time)
 {
     struct tm * timeinfo = localtime(&time);
@@ -430,6 +638,36 @@ void DateObject::exportXml(ticpp::Element* pConfig)
     pConfig->SetAttribute("type", "EIS4");
 }
 
+ObjectValue* DateObject::createObjectValue(const std::string& value)
+{
+    return new DateObjectValue(value);
+}
+
+bool DateObject::equals(ObjectValue* value)
+{
+    assert(value);
+    DateObjectValue* val = dynamic_cast<DateObjectValue*>(value);
+    if (val == 0)
+    {
+        std::cout << "DateObject: ERROR, equals() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+        return false;
+    }
+//    std::cout << "DateObject (id=" << getID() << "): Compare value_m='" << value_m << "' to value='" << val->value_m << "'" << std::endl;
+    return (day_m == val->day_m) && (month_m == val->month_m) && (year_m == val->year_m);
+}
+
+void DateObject::setValue(ObjectValue* value)
+{
+    assert(value);
+    DateObjectValue* val = dynamic_cast<DateObjectValue*>(value);
+    if (val == 0)
+        std::cout << "DateObject: ERROR, setValue() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+	if (val->day_m == -1)
+		setDate(time(0));
+	else        
+		setDate(val->day_m, val->month_m, val->year_m);	
+}
+
 void DateObject::onWrite(const uint8_t* buf, int len)
 {
     if (len < 5)
@@ -456,30 +694,6 @@ void DateObject::onWrite(const uint8_t* buf, int len)
     }
 }
 
-void DateObject::setIntValue(int value)
-{
-    if (value < 0)
-        setDate(time(0));
-    else
-    {
-        int rem = value;
-        int day = rem % 32;
-        rem = (rem-day) / 32; 
-        int month = rem % 13;
-        rem = (rem-month) / 13; 
-        int year = rem % 256;
-
-        setDate(day, month, year);
-    }
-}
-
-int DateObject::getIntValue()
-{
-    if (!init_m)
-        read();
-    return (year_m*13 + month_m) * 32 + day_m;
-}
-
 void DateObject::setDate(time_t time)
 {
     struct tm * timeinfo = localtime(&time);
@@ -526,6 +740,33 @@ void ValueObject::exportXml(ticpp::Element* pConfig)
     pConfig->SetAttribute("type", "EIS5");
 }
 
+ObjectValue* ValueObject::createObjectValue(const std::string& value)
+{
+    return new ValueObjectValue(value);
+}
+
+bool ValueObject::equals(ObjectValue* value)
+{
+    assert(value);
+    ValueObjectValue* val = dynamic_cast<ValueObjectValue*>(value);
+    if (val == 0)
+    {
+        std::cout << "ValueObject: ERROR, equals() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+        return false;
+    }
+    std::cout << "ValueObject (id=" << getID() << "): Compare value_m='" << value_m << "' to value='" << val->value_m << "'" << std::endl;
+    return value_m == val->value_m; 
+}
+
+void ValueObject::setValue(ObjectValue* value)
+{
+    assert(value);
+    ValueObjectValue* val = dynamic_cast<ValueObjectValue*>(value);
+    if (val == 0)
+        std::cout << "ValueObject: ERROR, setValue() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+	setFloatValue(val->value_m);	
+}
+
 void ValueObject::onWrite(const uint8_t* buf, int len)
 {
     if (len < 4)
@@ -601,6 +842,33 @@ void ScalingObject::exportXml(ticpp::Element* pConfig)
     pConfig->SetAttribute("type", "EIS6");
 }
 
+ObjectValue* ScalingObject::createObjectValue(const std::string& value)
+{
+    return new ScalingObjectValue(value);
+}
+
+bool ScalingObject::equals(ObjectValue* value)
+{
+    assert(value);
+    ScalingObjectValue* val = dynamic_cast<ScalingObjectValue*>(value);
+    if (val == 0)
+    {
+        std::cout << "ScalingObject: ERROR, equals() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+        return false;
+    }
+    std::cout << "ScalingObject (id=" << getID() << "): Compare value_m='" << value_m << "' to value='" << val->value_m << "'" << std::endl;
+    return value_m == val->value_m; 
+}
+
+void ScalingObject::setValue(ObjectValue* value)
+{
+    assert(value);
+    ScalingObjectValue* val = dynamic_cast<ScalingObjectValue*>(value);
+    if (val == 0)
+        std::cout << "ScalingObject: ERROR, setValue() received invalid class object (typeid=" << typeid(*value).name() << ")" << std::endl;
+	setIntValue(val->value_m);	
+}
+
 void ScalingObject::onWrite(const uint8_t* buf, int len)
 {
     int newValue;
