@@ -35,6 +35,13 @@ public:
     virtual void onChange(Object* object) = 0;
 };
 
+class ObjectValue
+{
+public:
+    virtual ~ObjectValue() {};
+    virtual std::string toString() = 0;
+};
+
 class Object
 {
 public:
@@ -44,6 +51,10 @@ public:
     static Object* create(ticpp::Element* pConfig);
     static Object* create(const std::string& type);
 
+    virtual ObjectValue* createObjectValue(const std::string& value) = 0;
+    virtual bool equals(ObjectValue* value) = 0;
+    virtual void setValue(ObjectValue* value) = 0;
+
     virtual void importXml(ticpp::Element* pConfig);
     virtual void updateXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
@@ -81,6 +92,10 @@ public:
     SwitchingObject();
     virtual ~SwitchingObject();
 
+    virtual ObjectValue* createObjectValue(const std::string& value);
+    virtual bool equals(ObjectValue* value);
+    virtual void setValue(ObjectValue* value);
+
     virtual void onWrite(const uint8_t* buf, int len);
     virtual void setIntValue(int value) { setBoolValue(value != 0); };
     virtual void setFloatValue(float value) { setBoolValue(value != 0); };
@@ -97,6 +112,10 @@ public:
     DimmingObject();
     virtual ~DimmingObject();
 
+    virtual ObjectValue* createObjectValue(const std::string& value);
+    virtual bool equals(ObjectValue* value);
+    virtual void setValue(ObjectValue* value);
+
     virtual void exportXml(ticpp::Element* pConfig);
 
     virtual void onWrite(const uint8_t* buf, int len);
@@ -112,13 +131,15 @@ public:
     TimeObject();
     virtual ~TimeObject();
 
+    virtual ObjectValue* createObjectValue(const std::string& value);
+    virtual bool equals(ObjectValue* value);
+    virtual void setValue(ObjectValue* value);
+
     virtual void exportXml(ticpp::Element* pConfig);
 
     virtual void onWrite(const uint8_t* buf, int len);
-    virtual void setIntValue(int value);
     virtual void setTime(time_t time);
     virtual void setTime(int wday, int hour, int min, int sec);
-    virtual int getIntValue();
 private:
     int wday_m;
     int hour_m;
@@ -132,13 +153,15 @@ public:
     DateObject();
     virtual ~DateObject();
 
+    virtual ObjectValue* createObjectValue(const std::string& value);
+    virtual bool equals(ObjectValue* value);
+    virtual void setValue(ObjectValue* value);
+
     virtual void exportXml(ticpp::Element* pConfig);
 
     virtual void onWrite(const uint8_t* buf, int len);
-    virtual void setIntValue(int value);
     virtual void setDate(time_t time);
     virtual void setDate(int day, int month, int year);
-    virtual int getIntValue();
 private:
     int day_m;
     int month_m;
@@ -151,6 +174,10 @@ public:
     ValueObject();
     virtual ~ValueObject();
 
+    virtual ObjectValue* createObjectValue(const std::string& value);
+    virtual bool equals(ObjectValue* value);
+    virtual void setValue(ObjectValue* value);
+
     virtual void exportXml(ticpp::Element* pConfig);
 
     virtual void onWrite(const uint8_t* buf, int len);
@@ -170,6 +197,10 @@ public:
     ScalingObject();
     virtual ~ScalingObject();
 
+    virtual ObjectValue* createObjectValue(const std::string& value);
+    virtual bool equals(ObjectValue* value);
+    virtual void setValue(ObjectValue* value);
+
     virtual void exportXml(ticpp::Element* pConfig);
 
     virtual void onWrite(const uint8_t* buf, int len);
@@ -179,6 +210,77 @@ private:
     int value_m;
 };
 
+class SwitchingObjectValue : public ObjectValue
+{
+public:
+    SwitchingObjectValue(const std::string& value);
+    virtual ~SwitchingObjectValue() {};
+    virtual std::string toString();
+private:
+	friend class SwitchingObject;
+    bool value_m;
+};
+
+class DimmingObjectValue : public ObjectValue
+{
+public:
+    DimmingObjectValue(const std::string& value);
+    virtual ~DimmingObjectValue() {};
+    virtual std::string toString();
+private:
+	friend class DimmingObject;
+    int value_m;
+};
+
+class TimeObjectValue : public ObjectValue
+{
+public:
+    TimeObjectValue(const std::string& value);
+    virtual ~TimeObjectValue() {};
+    virtual std::string toString();
+private:
+	friend class TimeObject;
+    int wday_m;
+    int hour_m;
+    int min_m;
+    int sec_m;
+};
+
+class DateObjectValue : public ObjectValue
+{
+public:
+    DateObjectValue(const std::string& value);
+    virtual ~DateObjectValue() {};
+    virtual std::string toString();
+private:
+	friend class DateObject;
+    int day_m;
+    int month_m;
+    int year_m;
+};
+
+class ValueObjectValue : public ObjectValue
+{
+public:
+    ValueObjectValue(const std::string& value);
+    virtual ~ValueObjectValue() {};
+    virtual std::string toString();
+private:
+	friend class ValueObject;
+    float value_m;
+};
+
+class ScalingObjectValue : public ObjectValue
+{
+public:
+    ScalingObjectValue(const std::string& value);
+    virtual ~ScalingObjectValue() {};
+    virtual std::string toString();
+private:
+	friend class ScalingObject;
+    int value_m;
+};
+
 class ObjectController : public TelegramListener
 {
 public:
