@@ -150,8 +150,10 @@ void Rule::updateXml(ticpp::Element* pConfig)
     ActionsList_t::iterator it;
     for(it=actionsList_m.begin(); it != actionsList_m.end(); ++it)
       delete (*it);
+    actionsList_m.clear();
     for(it=actionsListFalse_m.begin(); it != actionsListFalse_m.end(); ++it)
       delete (*it);
+    actionsListFalse_m.clear();
 
     ticpp::Iterator<ticpp::Element> actionListIt("actionlist");
     for ( actionListIt = pActionList; actionListIt != actionListIt.end(); actionListIt++ )
@@ -244,8 +246,8 @@ Action* Action::create(const std::string& type)
     return new DimUpAction();
   else if (type == "set-value")
     return new SetValueAction();
-//  else if (type == "switch")
-//    return new SwitchAction();
+  else if (type == "cycle-on-off")
+    return new CycleOnOffAction();
   else if (type == "send-sms")
     return new SendSmsAction();
   else
@@ -288,6 +290,12 @@ void DimUpAction::importXml(ticpp::Element* pConfig)
     std::string id;
     id = pConfig->GetAttribute("id");
     object_m = ObjectController::instance()->getObject(id);
+	if (!dynamic_cast<ScalingObject*>(object_m))
+	{
+		std::stringstream msg;
+		msg << "Wrong Object type for DimUpAction: '" << id << "'" << std::endl;
+    	throw ticpp::Exception(msg.str());
+	}
     pConfig->GetAttribute("start", &start_m);
     pConfig->GetAttribute("stop", &stop_m);
     pConfig->GetAttribute("duration", &duration_m);
@@ -345,6 +353,8 @@ SetValueAction::SetValueAction() : value_m(0), object_m(0)
 
 SetValueAction::~SetValueAction()
 {
+	if (value_m)
+		delete value_m;
 }
 
 void SetValueAction::importXml(ticpp::Element* pConfig)
@@ -356,7 +366,9 @@ void SetValueAction::importXml(ticpp::Element* pConfig)
     std::string value;
     value = pConfig->GetAttribute("value");
 
-    std::istringstream val(value);
+    value_m = object_m->createObjectValue(value);
+
+/*    std::istringstream val(value);
     val >> value_m;
 
     if ( val.fail() )
@@ -380,14 +392,15 @@ void SetValueAction::importXml(ticpp::Element* pConfig)
             throw ticpp::Exception(msg.str());
         }
     }
-    std::cout << "SetValueAction: Configured for object " << object_m->getID() << " with value " << value_m << std::endl;
+*/
+    std::cout << "SetValueAction: Configured for object " << object_m->getID() << " with value " << value_m->toString() << std::endl;
 }
 
 void SetValueAction::exportXml(ticpp::Element* pConfig)
 {
     pConfig->SetAttribute("type", "set-value");
     pConfig->SetAttribute("id", object_m->getID());
-    pConfig->SetAttribute("value", value_m);
+    pConfig->SetAttribute("value", value_m->toString());
 
     Action::exportXml(pConfig);
 }
@@ -395,9 +408,58 @@ void SetValueAction::exportXml(ticpp::Element* pConfig)
 void SetValueAction::Run (pth_sem_t * stop)
 {
     pth_sleep(delay_m);
-    std::cout << "Execute SetValueAction with value " << value_m << std::endl;
+    std::cout << "Execute SetValueAction with value " << value_m->toString() << std::endl;
     if (object_m)
-        object_m->setFloatValue(value_m);
+        object_m->setValue(value_m);
+}
+
+CycleOnOffAction::CycleOnOffAction() : object_m(0), count_m(0), delayOn_m(0), delayOff_m(0)
+{
+}
+
+CycleOnOffAction::~CycleOnOffAction()
+{
+}
+
+void CycleOnOffAction::importXml(ticpp::Element* pConfig)
+{
+    std::string id;
+    id = pConfig->GetAttribute("id");
+    object_m = ObjectController::instance()->getObject(id);
+
+    pConfig->GetAttribute("on", &delayOn_m);
+    pConfig->GetAttribute("off", &delayOff_m);
+    pConfig->GetAttribute("count", &count_m);
+    std::cout << "CycleOnOffAction: Configured for object " << object_m->getID()
+              << " with delay_on=" << delayOn_m
+              << "; delay_off=" << delayOff_m
+              << "; count=" << count_m << std::endl;
+}
+
+void CycleOnOffAction::exportXml(ticpp::Element* pConfig)
+{
+    pConfig->SetAttribute("type", "cycle-on-off");
+    pConfig->SetAttribute("id", object_m->getID());
+    pConfig->SetAttribute("on", delayOn_m);
+    pConfig->SetAttribute("off", delayOff_m);
+    pConfig->SetAttribute("count", count_m);
+
+    Action::exportXml(pConfig);
+}
+
+void CycleOnOffAction::Run (pth_sem_t * stop)
+{
+    if (!object_m)
+        return;
+    pth_sleep(delay_m);
+    std::cout << "Execute CycleOnOffAction" << std::endl;
+    for (int i=0; i<count_m; i++)
+    {
+        object_m->setBoolValue(true);
+        pth_sleep(delayOn_m);
+        object_m->setBoolValue(false);
+        pth_sleep(delayOff_m);
+    }
 }
 
 SendSmsAction::SendSmsAction()
@@ -580,22 +642,21 @@ void NotCondition::exportXml(ticpp::Element* pConfig)
   }
 }
   
-ObjectCondition::ObjectCondition(ChangeListener* cl) : cl_m(cl), trigger_m(false)
+ObjectCondition::ObjectCondition(ChangeListener* cl) : value_m(0), cl_m(cl), trigger_m(false)
 {
 }
 
 ObjectCondition::~ObjectCondition()
 {
+	if (value_m)
+		delete value_m;
 }
 
 bool ObjectCondition::evaluate()
 {
-    int value = object_m->getIntValue();
-    bool val = value == value_m;
+    bool val = object_m->equals(value_m);
     std::cout << "ObjectCondition (id='" << object_m->getID()
               << "') evaluated as '" << val
-              << "' value_m='" << value_m
-              << "' getIntValue='" << value
               << "'" << std::endl;
     return val;
 }
@@ -616,7 +677,10 @@ void ObjectCondition::importXml(ticpp::Element* pConfig)
 
     std::string value;
     value = pConfig->GetAttribute("value");
+    
+    value_m = object_m->createObjectValue(value);
 
+/*
     std::istringstream val(value);
     val >> value_m;
 
@@ -639,7 +703,8 @@ void ObjectCondition::importXml(ticpp::Element* pConfig)
             throw ticpp::Exception(msg.str());
         }
     }
-    std::cout << "ObjectCondition: configured value_m='" << value_m << "'" << std::endl;
+    */
+    std::cout << "ObjectCondition: configured value_m='" << value_m->toString() << "'" << std::endl;
 //    std::cout <<"(rdstate=" << (int)val.rdstate() << "; bad=" << val.bad() << "; good="<< val.good() << ")"<< std::endl;
 }
 
@@ -647,7 +712,7 @@ void ObjectCondition::exportXml(ticpp::Element* pConfig)
 {
   pConfig->SetAttribute("type", "object");
   pConfig->SetAttribute("id", object_m->getID());
-  pConfig->SetAttribute("value", value_m);
+  pConfig->SetAttribute("value", value_m->toString());
   if (trigger_m)
     pConfig->SetAttribute("trigger", "true");
 }
