@@ -1139,7 +1139,9 @@ bool TiXmlDocument::LoadFromString( const char* document, TiXmlEncoding encoding
 	rewind( fileBuf );
 
 	// Load FILE into TiXmlDocument
-	return LoadFile( fileBuf, encoding );
+	bool result = LoadFile( fileBuf, encoding );
+	fclose( fileBuf );
+	return result;
 }
 
 bool TiXmlDocument::LoadFile( const char* filename, TiXmlEncoding encoding )
